package org.rhq.sample.resourcefactory;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

import org.rhq.core.domain.measurement.AvailabilityType;
import org.rhq.core.domain.resource.CreateResourceStatus;
import org.rhq.core.pluginapi.inventory.CreateChildResourceFacet;
import org.rhq.core.pluginapi.inventory.CreateResourceReport;
import org.rhq.core.pluginapi.inventory.ResourceComponent;
import org.rhq.core.pluginapi.inventory.ResourceContext;

public class SamplePluginServerComponent implements ResourceComponent, CreateChildResourceFacet {
    private ResourceContext context;

    @Override
    public void start(ResourceContext context) {
        this.context = context;
    }

    @Override
    public void stop() {
    }

    @Override
    public AvailabilityType getAvailability() {
        return AvailabilityType.UP;
    }

    @Override
    public CreateResourceReport createResource(CreateResourceReport report) {
        String key = report.getPluginConfiguration().getSimpleValue("key", String.valueOf(System.currentTimeMillis()));
        report.setResourceKey(key);
        if (report.getUserSpecifiedResourceName() != null) {
            report.setResourceName(report.getUserSpecifiedResourceName());
        } else {
            report.setResourceName("My Child");
        }
        report.setStatus(CreateResourceStatus.SUCCESS);

        Properties childrenProps = readChildrenProperties();
        childrenProps.put(report.getResourceKey(), report.getResourceName());
        writeChildrenProperties(childrenProps);

        return report;
    }

    Properties readChildrenProperties() {
        File propsFile = getChildrenPropertiesFile();

        Properties childrenProps = new Properties();
        if (propsFile.exists()) {
            try {
                FileInputStream stream = new FileInputStream(propsFile);
                try {
                    childrenProps.load(stream);
                } finally {
                    stream.close();
                }
            } catch (Throwable t) {
                t.printStackTrace();
            }
        }
        return childrenProps;
    }

    void writeChildrenProperties(Properties childrenProps) {
        File propsFile = getChildrenPropertiesFile();
        propsFile.delete();

        try {
            FileOutputStream out = new FileOutputStream(propsFile);
            try {
                childrenProps.store(out, "child keys and names");
            } finally {
                out.close();
            }
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }

    File getChildrenPropertiesFile() {
        File dataDir = context.getDataDirectory();
        dataDir.mkdirs();
        File propsFile = new File(dataDir, "children.properties");
        return propsFile;
    }
}
